# Name: libsetuproi/command.py
# Purpose: Application commands, supporting Undo/Redo
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

__all__ = ['Command', 'CommandError', 'CommandRunner']


class Command:
    """
    Documents the Command interface.

    @type DESCRIPTION: str
    @cvar DESCRIPTION: message for use in undo menus (e.g. "Paste text")
    """

    DESCRIPTION = None

    def execute(self):
        """
        Performs the command, optionally recording previous state.

        @rtype:  object
        @return: memento which can undo the effects of this command, or None if
        the command cannot be undone.
        """

    def undo(self, memento):
        """
        Undoes the effects of a previous call to I{execute()}.
        """


class CommandError(Exception):
    """
    Signals the failure of a command.
    """

    def __init__(self, type, message, *args):
        """
        @type  type: str
        @param type: error type that may be used in a dialog box title

        @type  message: str
        @param message: description of the error that occured

        @type  args: varargs tuple
        @param args: I{message} is interpolated with this tuple
        """
        self.type = type
        try:
            self.message = message % args
        except Exception:
            self.message = message


class CommandRunner(object):
    """
    Manages command execution and undo history.

    @type dirty: bool
    @var  dirty: whether commands have modified things

    @type canUndo: bool
    @var  canUndo: whether the previous command may be undone

    @type canRestore: bool
    @var  canRestore: whether a previously undone command may be restored

    @type undoDescription: string or None
    @var  undoDescription: message for use in undo menus

    @type restoreDescription: string or None
    @var  restoreDescription: message for use in restore menus
    """

    def __init__(self):
        self.numModifications = 0
        self.curIdx = -1
        self.history = []

    def get_dirty(self):
        """
        Getter for the "dirty" property.
        """
        return self.numModifications != 0

    def set_dirty(self, dirty):
        """
        Setter for the "dirty" property.
        """
        if dirty:
            self.numModifications = len(self.history) or 1
        else:
            self.numModifications = 0

    dirty = property(get_dirty, set_dirty)

    def get_canUndo(self):
        """
        Getter for the "canUndo" property.
        """
        return -1 < self.curIdx and self.history[self.curIdx][1] is not None

    canUndo = property(get_canUndo)

    def get_canRestore(self):
        """
        Getter for the "canRestore" property.
        """
        return (-2 < self.curIdx
            and self.curIdx + 1 < len(self.history)
            and self.history[self.curIdx + 1][1] is None)

    canRestore = property(get_canRestore)

    def get_undoDescription(self):
        """
        Getter for the "undoDescription" property.
        """
        if self.canUndo:
            return self.history[self.curIdx][0].DESCRIPTION
        else:
            return None

    undoDescription = property(get_undoDescription)

    def get_restoreDescription(self):
        """
        Getter for the "restoreDescription" property.
        """
        if self.canRestore:
            return self.history[self.curIdx + 1][0].DESCRIPTION
        else:
            return None

    restoreDescription = property(get_restoreDescription)

    def _execute_command(self, command):
        """
        Private method to executes a command, handle execptions, and return a
        boolean status and memento object.
        """
        try:
            memento = command.execute()
        except CommandError, e:
            self.notify_failure(command, e)
            return False, None
##        except Exception, e:
##            self.notify_exception(command, e)
##            return False, None
        else:
            self.numModifications += 1
            return True, memento

    def execute(self, command):
        """
        Executes a command and returns a boolean indicating if it succeeded.
        """
        status, memento = self._execute_command(command)

        if not status:
            return False
        elif memento is None:
            self.curIdx = 0
            self.history = [[command, memento]]
        else:
            self.curIdx += 1
            self.history[self.curIdx:] = [[command, memento]]

        return True

    def undo(self):
        """
        Undoes a command and returns a boolean indicating if it succeeded.
        """
        if not self.canUndo:
            return False

        idx = self.curIdx
        command, memento = self.history[idx]

        try:
            command.undo(memento)
        except CommandError, e:
            self.notify_failure(command, e)
            return False
##        except Exception, e:
##            self.notify_exception(command, e)
##            return False

        self.curIdx -= 1
        self.numModifications -= 1
        self.history[idx][1] = None
        return True

    def restore(self):
        """
        Re-executes the previously undone command and returns a boolean
        indicating if it succeeded.
        """
        if not self.canRestore:
            return False

        idx = self.curIdx + 1
        command, memento = self.history[idx]
        status, memento = self._execute_command(command)

        if not status:
            del self.history[idx:]
            return False
        elif memento is None:
            self.curIdx = 0
            self.history = [[command, memento]]
        else:
            self.curIdx += 1
            self.history[idx][1] = memento

        return True

    def clearHistory(self):
        self.numModifications = 0
        self.curIdx = -1
        del self.history[:]

    def notify_failure(self, command, error):
        """
        Override to notify the user of a CommandError.
        """
#        print
#        print 'notify_failure(): [%s] %s' % (error.type, error.message)
#        print
        pass

##    def notify_exception(self, command, error):
##        """
##        Override to notify the user of an unhandled exception.
##        """
##        pass
