# Name: libsetuproi/presenter.py
# Purpose: Presenter for a setuproi frame
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import os.path
import libsetuproi.roicmds
from libsetuproi.roilist import *
from libsetuproi.spectra import *
from libsetuproi.command import Command, CommandRunner

__all__ = ['SetupRoiPresenter']


class BaseCommand(Command):
    def __init__(self, presenter):
        self.presenter = presenter
        self.selection = presenter.currentSelection[:]
        self.roiCmd = None

    def execute(self):
        memento = self.roiCmd.execute()
        viewMemento = self.update_view_for_execute()
        return memento, viewMemento

    def undo(self, memento):
        cmdMemento, viewMemento = memento
        self.roiCmd.undo(cmdMemento)
        self.update_view_for_undo(viewMemento)

    def update_view_for_execute(self):
        pass

    def update_view_for_undo(self, memento):
        pass


class ModifyRoiCommand(BaseCommand):
    DESCRIPTION = 'Modify ROI'

    def __init__(self, presenter, roi):
        BaseCommand.__init__(self, presenter)
        region, channel, column = presenter.currentSelection
        self.roiCmd = libsetuproi.roicmds.ModifyRoiCommand(presenter.roilist,
            region, channel, roi[0], roi[1])

    def _update_view(self):
        self.presenter.update_roi(self.selection[0], self.selection[1])
        self.presenter.set_selection(self.selection, replot=True)

    def update_view_for_execute(self):
        self._update_view()

    def update_view_for_undo(self, memento):
        self._update_view()


class LoadRoiCommand(BaseCommand):
    DESCRIPTION = 'Load ROI list'

    def __init__(self, presenter, fileName):
        BaseCommand.__init__(self, presenter)
        self.fileName = fileName
        self.roiCmd = libsetuproi.roicmds.LoadRoiCommand(presenter.roilist,
            fileName)

    def update_view_for_execute(self):
        oldFileName = self.presenter.fileName
        self._update_view(self.fileName)
        return oldFileName

    def update_view_for_undo(self, memento):
        self._update_view(memento)

    def _update_view(self, fileName):
        p = self.presenter
        p.update_roi_editor(True)
        p.set_file_name(fileName)


class ViewCommandRunner(CommandRunner):
    def __init__(self, view):
        CommandRunner.__init__(self)
        self.view = view

    def notify_failure(self, command, error):
        self.view.showErrorMessage(error.type, error.message)


class SetupRoiPresenter:
    """
    Encapsulates the user-interaction logic of setuproi's main window,
    following the Humble Dialog Box pattern proposed by Michael Feathers:
    U{http://www.objectmentor.com/resources/articles/TheHumbleDialogBox.pdf}
    """

    def __init__(self, director, view):
        self.director = director
        self.view = view
        self.fileName = None
        self.roilist = RoiList()
        self.cmdRunner = ViewCommandRunner(view)
        self.currentSelection = [0, 0, 0]

        nullSpectra = Spectra(None, None)
        self.spectraCache = {None: nullSpectra}
        self.preEdgeSpectra = self.postEdgeSpectra = nullSpectra

    def execute_command(self, command):
        status = self.cmdRunner.execute(command)
        self.update_undo_restore()
        return status

    def update_undo_restore(self):
        self.view.updateUndoRestore(self.cmdRunner.undoDescription,
            self.cmdRunner.restoreDescription)

    def update_roi(self, region, channel, roi=None):
        def to_string(x):
            if x is None: return ''
            else: return str(x)

        if roi is None:
            roi = self.roilist.getROI(region, channel)
        roistr = tuple([to_string(x) for x in roi])
        self.view.setRoi(region, channel, roistr)

    def replot_current_roi(self):
        region, channel, column = self.currentSelection
        roi = self.roilist.getROI(region, channel)
        self.view.setPlotSelection(region, channel)
        self.view.setPlotRoiRange(roi)
        self.view.updatePlot()

    def update_pre_edge_spectrum(self, channel):
        self.view.setPreEdgeSpectrum(
            self.get_spectrum(self.preEdgeSpectra, self.currentSelection[1]))

    def update_post_edge_spectrum(self, channel):
        self.view.setPostEdgeSpectrum(
            self.get_spectrum(self.postEdgeSpectra, self.currentSelection[1]))

    def get_spectrum(self, spectra, index):
        channel = self.currentSelection[1]
        if spectra is None or len(spectra.channels) <= channel:
            return None
        return spectra.channels[channel]

    def set_selection(self, selection, replot=False):
        region, channel, column = selection
        self.view.setRegionSelection(region)
        self.view.setChannelSelection(channel, column)
        self.selectRegion(region, replot=False)
        self.selectChannel(channel, column, replot=replot)

    # Closing the view

    def exitApplication(self):
        return self.director.exitApplication()

    def canClose(self):
        return not (self.cmdRunner.dirty and self.canSave())

    def closeFrame(self):
        if self.canClose() or not self.canSave():
            return True
        else:
            return self.view.askOkCancel('', 'Close without saving?')

    def destroyFrame(self):
        self.director.removeFrame(self.view)

    # Command Undo/Redo

# TODO: delete this
#    def clearUndoHistory(self):
#        self.cmdRunner.clearHistory()
#        self.update_undo_restore()

    def undoLastCommand(self):
        self.cmdRunner.undo()
        self.update_undo_restore()

    def restoreLastCommand(self):
        self.cmdRunner.restore()
        self.update_undo_restore()

    # File menu

    def canSave(self):
        return not self.roilist.isEmpty()

    def newRoiList(self):
        self.director.createNewFrame()

    def openRoiList(self):
        fileName = self.view.askFileOpen('Open ROI Script',
            directory=self.get_default_directory())

        if fileName is not None:
            clean = not self.cmdRunner.dirty
            if self.execute_command(LoadRoiCommand(self, fileName)) and clean:
                self.cmdRunner.dirty = False

    def saveRoiList(self):
        if self.fileName is None:
            self.saveRoiListAs()
        else:
            self.save_roi_list_to_file(self.fileName)

    def saveRoiListAs(self):
        fileName = self.view.askFileSave('Save ROI Script',
            directory=self.get_default_directory())

        if fileName is not None:
            self.save_roi_list_to_file(fileName)

    def get_default_directory(self):
        if self.fileName is None:
            return ''
        else:
            return os.path.dirname(self.fileName)

    def save_roi_list_to_file(self, fileName):
        script = self.roilist.toMotorScript()
        try:
            output = file(fileName, 'w')
            output.write(script)
            output.close()
        except IOError, e:
            if e.strerror:
                errmsg = e.strerror
            else:
                errmsg = str(e)

            self.view.showErrorMessage('I/O Error',
                'Error writing ROI script to `%s\':\n%s',
                os.path.basename(fileName), errmsg)
        else:
            self.set_file_name(fileName)
            self.cmdRunner.dirty = False

    def set_file_name(self, fileName):
        if fileName != self.fileName:
            self.fileName = fileName
            if fileName is None:
                self.view.setFileName(None)
            else:
                self.view.setFileName(os.path.basename(fileName))

    # ROI editor

    def selectRegion(self, region, replot=True):
        if region == self.currentSelection[0]:
            return

        self.currentSelection[0] = region
        self.update_roi_editor(replot)

    def update_roi_editor(self, replot):
        region = self.currentSelection[0]
        for channel, roi in enumerate(self.roilist.getChannelROIs(region)):
            self.update_roi(region, channel, roi)

        if replot:
            self.replot_current_roi()

    def selectChannel(self, channel, column, replot=False):
        region, oldChannel, oldColumn = self.currentSelection
        self.currentSelection[1:] = channel, column
        if oldChannel != channel:
            self.update_roi(region, channel)
            self.update_pre_edge_spectrum(channel)
            self.update_post_edge_spectrum(channel)
            self.replot_current_roi()
        elif replot:
            self.replot_current_roi()

    def updateRoiFromSelection(self, beginning, end):
        self.execute_command(ModifyRoiCommand(self, (beginning, end)))

    def editRoi(self, value):
        region, channel, column = self.currentSelection
        oldRoi = self.roilist.getROI(region, channel)
        newRoi = self._validate_roi_input(oldRoi, value)
        if newRoi is not None:
            self.execute_command(
                ModifyRoiCommand(self, newRoi))

    def _validate_roi_input(self, oldRoi, value):
        region, channel, column = self.currentSelection
        def revert_roi(message):
            self.view.showErrorMessage('Input Error', message)
            self.update_roi(region, channel, oldRoi)

        newRoi = list(oldRoi)
        value = value.strip()

        if not value:
            newRoi[column] = None
            return newRoi

        try:
            value = float(value.strip())
        except ValueError:
            revert_roi('Please enter a number.')
            return None

        if value < 0:
            revert_roi('Please enter a positive number.')
            return None
        else:
            newRoi[column] = value

        if newRoi[0] is not None and newRoi[1] is not None:
            if newRoi[1] < newRoi[0]:
                revert_roi('An ROI\'s beginning must come before it\'s end.')
                return None
            elif newRoi[0] == newRoi[1]:
                revert_roi('An ROI\'s beginning must differ from it\'s end.')
                return None
        return newRoi

    # Selecting which spectra to plot

    def selectPreEdgeSpectra(self, fileName):
        if fileName == self.preEdgeSpectra.fileName:
            return

        spectra = self.load_spectra_from_file(fileName)
        self.preEdgeSpectra = spectra
        self.update_pre_edge_spectrum(self.currentSelection[1])
        self.replot_current_roi()

    def selectPostEdgeSpectra(self, fileName):
        if fileName == self.postEdgeSpectra.fileName:
            return

        spectra = self.load_spectra_from_file(fileName)
        self.postEdgeSpectra = spectra
        self.update_post_edge_spectrum(self.currentSelection[1])
        self.replot_current_roi()

    def load_spectra_from_file(self, fileName):
        if fileName in self.spectraCache:
            return self.spectraCache[fileName]

        try:
            spectra = Spectra.fromFile(fileName)
        except IOError, e:
            if e.strerror:
                errmsg = e.strerror
            else:
                errmsg = str(e)

            self.view.showErrorMessage('I/O Error',
                'Error loading spectra from `%s\':\n%s',
                os.path.basename(fileName), errmsg)
            spectra = self.spectraCache[None]

        self.spectraCache[fileName] = spectra
        return spectra
