# Name: libsetuproi/roigrid.py
# Purpose: wx.Grid subclass which presents the ROI ranges
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import wx
import wx.grid

__all__ = ['RoiGrid']


class RoiGrid(wx.grid.Grid):
    def __init__(self, parent, id, **kwds):
        wx.grid.Grid.__init__(self, parent, id, **kwds)
        self.cellEditor = None

        self.CreateGrid(0, 2)
        self.EnableDragColSize(False)
        self.EnableDragRowSize(False)
        self.EnableDragGridSize(False)
        self.SetColLabelValue(0, 'Start')
        self.SetColLabelValue(1, 'Stop')
        self.SetLabelFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        self.AutoSizeLabels()

        wx.EVT_KEY_DOWN(self, self.OnKeyDown)
        wx.grid.EVT_GRID_LABEL_LEFT_CLICK(self, self.OnSelectGridLabel)

    def OnSelectGridLabel(self, evt):
        # ignore the column label clicks
        if evt.GetCol() != -1:
            return

        if self.IsCellEditControlEnabled():
            self.DisableCellEditControl()

        self.SetGridCursor(evt.GetRow(), 0)

    def AutoSizeLabels(self): 
        if self.GetNumberRows() == 0:
            self.SetRowLabelSize(0)
            self.SetColLabelSize(0)
            return

        dc = wx.ScreenDC() 
        dc.SetFont(self.GetLabelFont()) 

        maxWidth = 0 
        i = self.GetNumberRows() - 1 
        while i >= 0: 
            curWidth = dc.GetTextExtent("M%s" % (self.GetRowLabelValue(i)))[0] 
            if maxWidth < curWidth:
                maxWidth = curWidth 
            i -= 1
        self.SetRowLabelSize(maxWidth) 

        maxHeight = 0 
        i = self.GetNumberCols() - 1 
        while i >= 0: 
            (w,h,d,l) = dc.GetFullTextExtent(self.GetColLabelValue(i)) 
            curHeight = h + d + l + 4 
            if maxHeight < curHeight:
                maxHeight = curHeight 
            i -= 1
        self.SetColLabelSize(maxHeight)

    def SetNumberRows(self, numRows):
        self.BeginBatch()
        currNumRows = self.GetNumberRows()
        if currNumRows < numRows:
            count  = numRows - currNumRows
            self.InsertRows(currNumRows, count)
            if self.cellEditor is None:
                self.cellEditor = wx.grid.GridCellNumberEditor()
            for i in range(currNumRows, currNumRows + count):
                self.SetRowLabelValue(i, str(i))
                self.SetCellEditor(i, 0, self.cellEditor)
                self.SetCellEditor(i, 1, self.cellEditor)
        elif numRows < currNumRows:
            n = currNumRows - numRows
            self.DeleteRows(currNumRows - n, n)
            self.cellEditor = None

        self.AutoSizeLabels()
        self.EndBatch()

    def OnKeyDown(self, evt):
        isEditing = self.IsCellEditControlEnabled()

        if (evt.KeyCode() in (wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER)
        and not evt.ControlDown()):
            # the edit control needs Ctrl+Return, so check for it
            if isEditing:
                self.DisableCellEditControl()
            else:
                self.EnableCellEditControl()
        else:
            # Tab presumably causes a call to SetGridCursor(), which ends the
            # editing after changing the selection.  The result is that if you
            # tab while editing an ROI start, the ROI end gets modified.  This
            # works around the problem.
            if evt.KeyCode() == wx.WXK_TAB and isEditing:
                self.DisableCellEditControl()
            evt.Skip()
