# Name: libsetuproi/view.py
# Purpose: setuproi's main window
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import wx
import wxmpl
import os.path
from libsetuproi.plot import *
from libsetuproi.roigrid import *
from libsetuproi.filechoice import *
from libsetuproi.togglebox import *
from libsetuproi.presenter import *
from libsetuproi.wxview import FrameView

__all__ = ['SetupRoiFrame']


class SetupRoiFrame(FrameView):
    windowName = dialogName = 'setuproi'

    def __init__(self, parent, director, **kwds):
        kwargs = {'size': wx.Size(1000, 625)}
        kwargs.update(kwds)
        FrameView.__init__(self, parent, **kwargs)

        self.fileName = None
        self.roiFileName = None
        self.undoDescription = None
        self.restoreDescription = None
        self.selectedRoiToggle = None
        self.selectingRoiFromPlot = False

        # Calling SetGridCursor() within EVT_GRID_CELL_CHANGE fires off another
        # EVT_GRID_CELL_CHANGE event (because it disabled the edit control).
        # This is caught in OnChangeEditorField() and the event is Veto()'d.
        self.handlingCellChange = False

        # Calling SetGridCursor() from setChannelSelection() results in a
        # EVT_GRID_SELECT_CELL event that the presenter doesn't want to know
        # about.
        self.ignoreGridSelection = False

        self.create_widgets()
        self.bind_events()
        self.presenter = SetupRoiPresenter(director, self)

    def bind_events(self):
        # Window lifecycle events
        wx.EVT_WINDOW_DESTROY(self, self.OnWindowDestroy)
        wx.EVT_CLOSE(self, self.OnClose)

        # Selection events
        EVT_FILE_CHOICE(self, self.preEdgeChoice.GetId(), self.OnSelectPreEdge)
        EVT_FILE_CHOICE(self, self.postEdgeChoice.GetId(),
            self.OnSelectPostEdge)
        EVT_TOGGLEBOX(self, self.roiToggles.GetId(), self.OnSelectRegion)
        wx.grid.EVT_GRID_SELECT_CELL(self.channelEditor,
            self.OnSelectEditorField)

        # Modification events
        wx.EVT_BUTTON(self, self.clearRoiButton.GetId(), self.OnClearRoi)
        wx.grid.EVT_GRID_CELL_CHANGE(self.channelEditor,
            self.OnChangeEditorField)
        wxmpl.EVT_POINT(self, self.roiPlot.GetId(), self.OnPlotPoint)

    def create_widgets(self):
        self.create_menu_bar()

        self.roiPlot = RoiPlotPanel(self)
        fpnl = self.create_file_panel()
        cpnl = self.create_editor_panel()

        # plot sizer
        plotSzr = wx.BoxSizer(wx.VERTICAL)
        plotSzr.Add(fpnl,         0, wx.EXPAND, 0)
        plotSzr.Add(self.roiPlot, 1, wx.EXPAND|wx.TOP, 2)

        # frame sizer
        s = wx.BoxSizer(wx.HORIZONTAL)
        s.Add(plotSzr, 1, wx.EXPAND|wx.TOP|wx.BOTTOM|wx.LEFT,  2)
        s.Add(cpnl,    0, wx.EXPAND|wx.TOP|wx.BOTTOM|wx.RIGHT, 2)
        self.SetSizer(s)

    def make_font_bold(self, widget):
        fnt = widget.GetFont()
        fnt.SetWeight(wx.BOLD)
        widget.SetFont(fnt)

    def create_file_panel(self):
        fpnl = wx.Panel(self, -1)

        preEdgeLabel = wx.StaticText(fpnl, -1, 'Pre-Edge')
        self.make_font_bold(preEdgeLabel)
        self.preEdgeChoice  = FileChoice(fpnl, -1)

        postEdgeLabel = wx.StaticText(fpnl, -1, 'Post-Edge')
        self.make_font_bold(postEdgeLabel)
        self.postEdgeChoice = FileChoice(fpnl, -1,
            shareDirectory=self.preEdgeChoice)

        s = wx.BoxSizer(wx.HORIZONTAL)
        s.Add(preEdgeLabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 2)
        s.Add(self.preEdgeChoice, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        s.Add((50, -1), 0, 0, 0)
        s.Add(postEdgeLabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 2)
        s.Add(self.postEdgeChoice, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        s.Add((50, -1), 0, 0, 0)
        fpnl.SetSizer(s)
        fpnl.Fit()
        return fpnl

    def create_editor_panel(self):
        ctrls = wx.Panel(self, -1, size=wx.Size(220, -1))
        roiLabel = wx.StaticText(ctrls, -1, 'Region of Interest')
        self.make_font_bold(roiLabel)
        self.roiToggles = ToggleBox(ctrls, -1, 2, 8)

        # region sizer
        roiSzr = wx.BoxSizer(wx.VERTICAL)
        roiSzr.Add(roiLabel, 0, wx.BOTTOM, 2)
        roiSzr.Add(self.roiToggles, 0, wx.ALIGN_CENTER, 0)

        # channel roi editor
        self.channelEditor = RoiGrid(ctrls, -1)
        self.channelEditor.SetNumberRows(19)

        # select roi button
        self.clearRoiButton = wx.Button(ctrls, -1, 'Clear ROI')
        self.clearRoiButton.SetFont(
            wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.NORMAL))

        # ctrls panel sizer
        s = wx.BoxSizer(wx.VERTICAL)
        s.Add(roiSzr, 0, wx.EXPAND|wx.ALL, 2)
        s.Add(self.channelEditor, 1, wx.EXPAND|wx.ALL, 2)
        s.Add(self.clearRoiButton, 0, wx.EXPAND|wx.LEFT, 2)
        ctrls.SetSizer(s)
        return ctrls

    def create_menu_bar(self):
        MENUS = (
            ('&File', self.make_file_menu),
            ('&Edit', self.make_edit_menu)
        )

        mbar = wx.MenuBar()
        self.SetMenuBar(mbar)

        for (title, creator) in MENUS:
            menu  = wx.Menu()
            creator(menu)
            mbar.Append(menu, title)

    def make_file_menu(self, menu):
        menu.Append(wx.ID_NEW,    '&New\tCtrl+N')
        menu.Append(wx.ID_OPEN,   '&Open...\tCtrl+O')
        menu.Append(wx.ID_CLOSE,  '&Close\tCtrl+W')
        menu.AppendSeparator()
        menu.Append(wx.ID_SAVE,   '&Save\tCtrl+S')
        menu.Append(wx.ID_SAVEAS, 'Save &As...\tCtrl+A')
        menu.AppendSeparator()
        menu.Append(wx.ID_EXIT,   '&Quit')

        def M(id, function):
            wx.EVT_MENU(self, id, function)
        M(wx.ID_NEW,    self.OnFileNew)
        M(wx.ID_OPEN,   self.OnFileOpen)
        M(wx.ID_CLOSE,  self.OnFileClose)
        M(wx.ID_SAVE,   self.OnFileSave)
        M(wx.ID_SAVEAS, self.OnFileSaveAs)
        M(wx.ID_EXIT,   self.OnFileQuit)

        def U(id, function):
            wx.EVT_UPDATE_UI(self, id, function)
        U(wx.ID_SAVE, self.OnUpdateFileSave)
        U(wx.ID_SAVEAS, self.OnUpdateFileSaveAs)

    def make_edit_menu(self, menu):
        menu.Append(wx.ID_UNDO, 'Undo\tCtrl+Z')
        menu.Append(wx.ID_REDO, 'Restore\tCtrl+Y')

        def M(id, function):
            wx.EVT_MENU(self, id, function)
        M(wx.ID_UNDO,  self.OnEditUndo)
        M(wx.ID_REDO,  self.OnEditRestore)

        def U(id, function):
            wx.EVT_UPDATE_UI(self, id, function)
        U(wx.ID_UNDO,  self.OnUpdateEditUndo)
        U(wx.ID_REDO,  self.OnUpdateEditRedo)

    # MenuEvent handlers

    def OnFileNew(self, evt):
        self.presenter.newRoiList()

    def OnFileOpen(self, evt):
        self.presenter.openRoiList()

    def OnFileClose(self, evt):
        self.Close()

    def OnFileSave(self, evt):
        self.presenter.saveRoiList()

    def OnFileSaveAs(self, evt):
        self.presenter.saveRoiListAs()

    def OnFileQuit(self, evt):
        self.presenter.exitApplication()

    def OnEditUndo(self, evt):
        self.presenter.undoLastCommand()

    def OnEditRestore(self, evt):
        self.presenter.restoreLastCommand()

    # UpdateUIEvent handlers

    def OnUpdateFileSave(self, evt):
        evt.Enable(self.presenter.canSave())

    def OnUpdateFileSaveAs(self, evt):
        evt.Enable(self.presenter.canSave())

    def OnUpdateEditUndo(self, evt):
        evt.Enable(self.undoDescription is not None)
        if self.undoDescription is None:
            evt.SetText('Undo')
        else:
            evt.SetText('Undo ' + self.undoDescription)

    def OnUpdateEditRedo(self, evt):
        evt.Enable(self.restoreDescription is not None)
        if self.restoreDescription is None:
            evt.SetText('Restore')
        else:
            evt.SetText('Restore ' + self.restoreDescription)

    # Window lifecycle event handlers

    def OnWindowDestroy(self, evt):
        self.presenter.destroyFrame()

    def OnClose(self, evt):
        if evt.CanVeto() and not self.presenter.closeFrame():
            evt.Veto()
        evt.Skip()

    # Selection Events

    def OnSelectPreEdge(self, evt):
        self.presenter.selectPreEdgeSpectra(evt.fileName)

    def OnSelectPostEdge(self, evt):
        self.presenter.selectPostEdgeSpectra(evt.fileName)

    def OnSelectRegion(self, evt):
        self.presenter.selectRegion(evt.GetSelection())

    def OnSelectEditorField(self, evt):
        evt.Skip()
        if self.ignoreGridSelection:
            self.ignoreGridSelection = False
        else:
            self.presenter.selectChannel(evt.GetRow(), evt.GetCol())

    # Modification events

#    def OnSelectRoi(self, evt):
#        self.update_selecting_roi_from_plot(not self.selectingRoiFromPlot)

    def OnClearRoi(self, evt):
        channel = self.channelEditor.GetGridCursorRow()
        self.channelEditor.SetCellValue(channel, 0, '')
        self.channelEditor.SetCellValue(channel, 1, '')
        self.presenter.updateRoiFromSelection(None, None)

    def OnChangeEditorField(self, evt):
        if self.handlingCellChange:
            evt.Veto()
            return

        self.handlingCellChange = True
        value = self.channelEditor.GetCellValue(evt.GetRow(), evt.GetCol())
        self.presenter.editRoi(value)
        self.handlingCellChange = False

    def OnPlotPoint(self, evt):
        # FIXME: going from Xpt,ROI to X1,X2 should be handled by the presenter

        x1 = self.roiPlot.roiStart
        x2 = self.roiPlot.roiEnd

        if x1 is None:
            x1 = evt.xdata
        elif x2 is None:
            x1, x2 = min(x1, evt.xdata), max(x1, evt.xdata)
        elif evt.xdata <= x1 + abs(x2-x1)/2.0:
            x1 = evt.xdata
        else:
            x2 = evt.xdata

        channel = self.channelEditor.GetGridCursorRow()
        self.channelEditor.SetCellValue(channel, 0, str(x1))
        self.channelEditor.SetCellValue(channel, 1, str(x2))
        self.presenter.updateRoiFromSelection(x1, x2)

    # AppDirector API

    def canClose(self):
        return self.presenter.canClose()

    # SetupRoiPresenter API

    def updateUndoRestore(self, undoDescription, restoreDescription):
        self.undoDescription    = undoDescription
        self.restoreDescription = restoreDescription

    def setFileName(self, fileName):
        self.updateTitle(fileName)

    def setPreEdgeSpectrum(self, spectrum):
        self.roiPlot.preEdgeSpectrum = spectrum

    def setPostEdgeSpectrum(self, spectrum):
        self.roiPlot.postEdgeSpectrum = spectrum

    def setPlotSelection(self, region, channel):
        self.roiPlot.region = region
        self.roiPlot.channel = channel

    def setPlotRoiRange(self, roi):
        self.roiPlot.roiStart = roi[0]
        self.roiPlot.roiEnd   = roi[1]

    def updatePlot(self):
#        wx.BeginBusyCursor()
        self.roiPlot.redraw()
#        wx.EndBusyCursor()

    def setRoi(self, region, channel, roi):
        self.channelEditor.SetCellValue(channel, 0, roi[0])
        self.channelEditor.SetCellValue(channel, 1, roi[1])

    def setRegionSelection(self, region):
        self.roiToggles.SetSelection(region)

    def setChannelSelection(self, channel, column):
        self.ignoreGridSelection = True
        self.channelEditor.SetGridCursor(channel, column)
