#!/usr/bin/env python
# Name: test_appmodel.py
# Purpose: Unit tests for libsetuproi.appmodel.SetupRoiModel
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import unittest
from libsetuproi.roicmds import *
from libsetuproi.roilist import RoiList
from libsetuproi.command import CommandRunner


class TestModifyRoiCommand(unittest.TestCase):
    def test(self):
        mdl = RoiList()
        cr  = CommandRunner()
        r1 = mdl.getROI(0, 0)
        r2 = 1, 1

        self.assert_(cr.execute(ModifyRoiCommand(mdl, 0, 0, *r2)))
        self.assertEquals(r2, mdl.getROI(0, 0))
        cr.undo()
        self.assertEquals(r1, mdl.getROI(0, 0))
        cr.restore()
        self.assertEquals(r2, mdl.getROI(0, 0))


class TestLoadRoiCommand(unittest.TestCase):
    def test(self):
        rl = RoiList()
        cr = CommandRunner()
        import os
        import tempfile
        script = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')
        fd, fname = tempfile.mkstemp()
        os.write(fd, script)
        os.close(fd)

        try:
            self.assert_(cr.execute(LoadRoiCommand(rl, fname)))
            self.assertEquals((1, 2), rl.getROI(0, 0))
            self.assertEquals((2, 3), rl.getROI(0, 1))
            self.assertEquals((3, 4), rl.getROI(0, 2))
            cr.undo()
            self.assert_(rl.isEmpty(), 'rl should be empty')
            cr.restore()
            self.assertEquals((1, 2), rl.getROI(0, 0))
            self.assertEquals((2, 3), rl.getROI(0, 1))
            self.assertEquals((3, 4), rl.getROI(0, 2))
        finally:
            os.unlink(fname)


class TestLoadRoiCommandErrors(unittest.TestCase):
    def test_bad_file_name(self):
        rl = RoiList()
        cr = CommandRunner()
        self.assert_(not cr.execute(LoadRoiCommand(rl, 'bad.file')),
            'load should fail')
 
    def test_empty_file(self):
        rl = RoiList()
        cr = CommandRunner()
        import os
        import tempfile
        fd, fname = tempfile.mkstemp()
        os.close(fd)

        try:
            self.assert_(not cr.execute(LoadRoiCommand(rl, fname)),
                'load should fail')
        finally:
            os.unlink(fname)


if __name__ == '__main__':
    unittest.main()
